/*
Copyright (c) 2005 Aristotle Pagaltzis

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#define _GNU_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <err.h>

#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>

#if !(defined(LIBXML_XPATH_ENABLED) && defined(LIBXML_SAX1_ENABLED))
#	error "Your copy of libxml2 is not compiled with support for the necessary APIs (XPath and SAX1)"
#endif

int usage_error = 0;

xmlDocPtr doc = NULL;
xmlXPathContextPtr xpathctx = NULL;
xmlXPathObjectPtr xpathobj = NULL;

void
cleanup( void )
{
	if( usage_error ) fprintf( stderr, "usage: xpathgrep file.xml xpathexpr [ prefix namespace [ prefix namespace ... ] ]\n" "\n" );
	if( xpathobj ) xmlXPathFreeObject( xpathobj );
	if( xpathctx ) xmlXPathFreeContext( xpathctx );
	if( doc ) xmlFreeDoc( doc );
	xmlCleanupParser();
}

int
main( int argc, char **argv )
{
	char *filename = argv[1], *xpath = argv[2];
	int i;

	atexit( cleanup );

	if( argc < 3 ) return usage_error = 1;

	if( ( argc - 3 ) % 2 ) errx( usage_error = 2, "Odd number of elements in prefix/namespace pair list\n" );

	xmlInitParser();
	LIBXML_TEST_VERSION
		
	doc = xmlParseFile( filename );
	if( !doc ) errx( 2, "Error: unable to parse file: %s", filename );

	xpathctx = xmlXPathNewContext( doc );
	if( !xpathctx ) errx( -1, "Error: unable to create new XPath context" );

	for( i = 3; i < argc; i += 2 )
	{
		char * prefix  = argv[i];
		char * uri     = argv[i + 1];
		int    failure = xmlXPathRegisterNs( xpathctx, BAD_CAST prefix, BAD_CAST uri );
		if( failure ) errx( -1, "Error: unable to register NS with prefix \"%s\" bound to URI \"%s\"", prefix, uri );
	}

	xpathobj = xmlXPathEvalExpression( BAD_CAST xpath, xpathctx );
	if( !xpathobj ) errx( 3, "Error: unable to evaluate xpath expression: %s", xpath );

	{
		int size = ( xpathobj->nodesetval ) ? xpathobj->nodesetval->nodeNr : 0;

		/* Reserialise fragment by copying it into a new document */
		for( i = 0; i < size; ++i )
		{
			xmlDocPtr newdoc = xmlNewDoc( BAD_CAST "1.0" );
			xmlNodePtr copied = xmlDocCopyNode( xpathobj->nodesetval->nodeTab[i], newdoc, 1 );
			xmlDocSetRootElement( newdoc, copied );

			{
				xmlBufferPtr buf = xmlBufferCreate();
				xmlNodeDump( buf, newdoc, copied, 0, 0 );
				printf( "%s\n", ( char * ) xmlBufferContent( buf ) );
				xmlBufferFree( buf );
			}

			xmlFreeDoc( newdoc );
		}
	}

	return 0;
}
